'use client';

import { z } from 'zod';
import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import Link from 'next/link';
import { signIn, signUp } from '@/lib/queries';
import { useRouter } from 'next/navigation';
import { useEffect, useState } from 'react';
import { useAuth } from '@/lib/hooks/context';
import { useSupabaseClient } from '@/utils/supabase';
import { StatusMessage, SubmitButton } from '@/components/default/forms';
import {
  Card,
  CardContent,
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
  Input,
  Separator,
  Tabs,
  TabsContent,
  TabsList,
  TabsTrigger,
} from '@/components/ui';
import {
  SignInWithApple,
  type SignInWithAppleProps,
  SignInWithMicrosoft,
  type SignInWithMicrosoftProps,
} from '@/components/default/auth/buttons/client';
import { type ComponentProps } from 'react';
import { cn } from '@/lib/utils';

const signInFormSchema = z.object({
  email: z.string().email({
    message: 'Please enter a valid email address.'
  }),
  password: z.string().min(8, {
    message: 'Password must be at least 8 characters.'
  }),
});

const signUpFormSchema = z
  .object({
    name: z.string().min(2, {
      message: 'Name must be at least 2 characters.',
    }),
    email: z.string().email({
      message: 'Please enter a valid email address.',
    }),
    password: z.string().min(8, {
      message: 'Password must be at least 8 characters.',
    }),
    confirmPassword: z.string().min(8, {
      message: 'Password must be at least 8 characters.',
    }),
  })
  .refine((data) => data.password === data.confirmPassword, {
    message: 'Passwords do not match!',
    path: ['confirmPassword'],
  });

type SignInCardProps = {
  containerClassName?: ComponentProps<typeof Card>['className'];
  containerProps?: Omit<ComponentProps<typeof Card>, 'className'>;
  tabsClassName?: ComponentProps<typeof Tabs>['className'];
  tabsProps?: Omit<ComponentProps<typeof Tabs>, 'className'>;
  tabsListClassName?: ComponentProps<typeof TabsList>['className'];
  tabsListProps?: Omit<ComponentProps<typeof TabsList>, 'className'>;
  tabsTriggerClassName?: ComponentProps<typeof TabsTrigger>['className'];
  tabsTriggerProps?: Omit<ComponentProps<typeof TabsTrigger>, 'className' | 'value'>;
  cardClassName?: ComponentProps<typeof Card>['className'];
  cardProps?: Omit<ComponentProps<typeof Card>, 'className'>;
  formClassName?: ComponentProps<'form'>['className'];
  formProps?: Omit<ComponentProps<'form'>, 'className' | 'onSubmit'>;
  formLabelClassName?: ComponentProps<typeof FormLabel>['className'];
  formLabelProps?: Omit<ComponentProps<typeof FormLabel>, 'className'>;
  submitButtonProps?: Omit<ComponentProps<typeof SubmitButton>,
    'pendingText' | 'disabled'>;
  signInWithAppleProps?: SignInWithAppleProps;
  signInWithMicrosoftProps?: SignInWithMicrosoftProps;
};

export const SignInCard = ({
  containerClassName,
  containerProps,
  tabsClassName,
  tabsProps = { defaultValue: 'sign-in' },
  tabsListClassName,
  tabsListProps,
  tabsTriggerClassName,
  tabsTriggerProps,
  cardClassName,
  cardProps,
  formClassName,
  formProps,
  formLabelClassName,
  formLabelProps,
  submitButtonProps,
  signInWithAppleProps,
  signInWithMicrosoftProps,
}: SignInCardProps) => {

  const router = useRouter();
  const { isAuthenticated, loading, refreshUser } = useAuth();
  const [statusMessage, setStatusMessage] = useState('');
  const supabase = useSupabaseClient();

  const signInForm = useForm<z.infer<typeof signInFormSchema>>({
    resolver: zodResolver(signInFormSchema),
    defaultValues: { email: '', password: '' },
  });

  const signUpForm = useForm<z.infer<typeof signUpFormSchema>>({
    resolver: zodResolver(signUpFormSchema),
    defaultValues: {
      name: '',
      email: '',
      password: '',
      confirmPassword: '',
    },
  });

  useEffect(() => {
    if (isAuthenticated) router.push('/');
  }, [isAuthenticated, router]);

  const handleSignIn = async (values: z.infer<typeof signInFormSchema>) => {
    try {
      setStatusMessage('');
      const formData = new FormData();
      formData.append('email', values.email);
      formData.append('password', values.password);
      if (!supabase) throw new Error('Supabase client not found');
      const result = await signIn(supabase, formData);
      if (result.error) throw new Error(result.error.message);
      await refreshUser();
      signInForm.reset();
      router.push('');
    } catch (error) {
      setStatusMessage(`Error signing in: ${error as string}`);
    }
  };

  const handleSignUp = async (values: z.infer<typeof signUpFormSchema>) => {
    try {
      setStatusMessage('');
      const formData = new FormData();
      formData.append('name', values.name);
      formData.append('email', values.email);
      formData.append('password', values.password);
      if (!supabase) throw new Error('Supabase client not found');
      const result = await signUp(supabase, formData);
      if (result.error) throw new Error(result.error.message);
      await refreshUser();
      signUpForm.reset();
      router.push('');
    } catch (error) {
      setStatusMessage(`Error signing up: ${error as string}`);
    }
  };

  return (
    <Card
      className={cn('p-4 bg-card/25 min-h-[720px]', containerClassName)}
      {...containerProps}
    >
      <Tabs
        className={cn('items-center', tabsClassName)}
        {...tabsProps}
      >
        <TabsList
          className={cn('py-6', tabsListClassName)}
          {...tabsListProps}
        >
          <TabsTrigger
            value='sign-in'
            className={cn(
              'p-6 text-2xl font-bold cursor-pointer',
              tabsTriggerClassName
            )}
            {...tabsTriggerProps}
          >
            Sign In
          </TabsTrigger>
          <TabsTrigger
            value='sign-up'
            className={cn(
              'p-6 text-2xl font-bold cursor-pointer',
              tabsTriggerClassName,
            )}
            {...tabsTriggerProps}
          >
            Sign Up
          </TabsTrigger>
        </TabsList>
        <TabsContent value='sign-in'>
          <Card
            className={cn(
              'min-w-xs sm:min-w-sm bg-card/50',
              cardClassName,
            )}
            {...cardProps}
          >
            <CardContent>
              <Form {...signInForm}>
                <form
                  onSubmit={signInForm.handleSubmit(handleSignIn)}
                  className={cn('flex flex-col space-y-6', formClassName)}
                  {...formProps}
                >
                  <FormField
                    control={signInForm.control}
                    name='email'
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel
                          className={cn('text-xl', formLabelClassName)}
                          {...formLabelProps}>
                          Email
                        </FormLabel>
                        <FormControl>
                          <Input
                            type='email'
                            placeholder='you@example.com'
                            {...field}
                          />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  <FormField
                    control={signInForm.control}
                    name='password'
                    render={({ field }) => (
                      <FormItem>
                        <div className='flex justify-between'>
                          <FormLabel
                            className={cn('text-xl', formLabelClassName)}
                            {...formLabelProps}
                          >
                            Password
                          </FormLabel>
                          <Link
                            href='/forgot-password'
                          >
                            Forgot Password?
                          </Link>
                        </div>
                        <FormControl>
                          <Input
                            type='password'
                            placeholder='Your password'
                            {...field}
                          />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  {statusMessage &&
                    (statusMessage.includes('Error') ||
                    statusMessage.includes('error') ||
                    statusMessage.includes('failed') ||
                    statusMessage.includes('invalid') ? (
                      <StatusMessage message={{ error: statusMessage }} />
                    ) : (
                      <StatusMessage message={{ message: statusMessage }} />
                  ))}
                  <SubmitButton
                    disabled={loading}
                    pendingText='Signing In...'
                    className={cn(
                      'text-lg font-semibold w-2/3 mx-auto',
                      submitButtonProps?.className
                    )}
                    {...submitButtonProps}
                  >
                    Sign In
                  </SubmitButton>
                </form>
              </Form>
              <div className='flex items-center w-full gap-4 mt-4'>
                <Separator className='flex-1 bg-muted-foreground/50 py-0.5' />
                <span className='text-muted-foreground'>or</span>
                <Separator className='flex-1 bg-muted-foreground/50 py-0.5' />
              </div>
              <SignInWithMicrosoft
                submitButtonProps = {{
                  className: cn(
                    'flex w-5/6 m-auto',
                    signInWithMicrosoftProps?.submitButtonProps?.className),
                }}
                textClassName={cn(
                  'text-lg',
                  signInWithMicrosoftProps?.textClassName,
                )}
                iconClassName={cn(
                  'size-6',
                  signInWithMicrosoftProps?.iconClassName,
                )}
                {...signInWithMicrosoftProps}
              />
              <SignInWithApple
                submitButtonProps = {{
                  className: cn(
                    'flex w-5/6 m-auto',
                    signInWithAppleProps?.submitButtonProps?.className),
                }}
                textClassName={cn(
                  'text-lg',
                  signInWithAppleProps?.textClassName,
                )}
                iconClassName={cn(
                  'size-6',
                  signInWithAppleProps?.iconClassName,
                )}
                {...signInWithAppleProps}
              />
            </CardContent>
          </Card>
        </TabsContent>
        <TabsContent value='sign-up'>
          <Card
            className={cn(
              'min-w-xs sm:min-w-sm bg-card/50',
              cardClassName,
            )}
            {...cardProps}
          >
            <CardContent>
              <Form {...signUpForm}>
                <form
                  className={cn('flex flex-col space-y-6', formClassName)}
                  onSubmit={signUpForm.handleSubmit(handleSignUp)}
                  {...formProps}
                >
                  <FormField
                    control={signUpForm.control}
                    name='name'
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel
                          className={cn('text-xl', formLabelClassName)}
                          {...formLabelProps}
                        >
                          Name
                        </FormLabel>
                        <FormControl>
                          <Input
                            type='text'
                            placeholder='Full Name'
                            {...field}
                          />
                        </FormControl>
                      </FormItem>
                    )}
                  />
                  <FormField
                    control={signUpForm.control}
                    name='email'
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel
                          className={cn('text-xl', formLabelClassName)}
                          {...formLabelProps}
                        >
                          Email
                        </FormLabel>
                        <FormControl>
                          <Input
                            type='email'
                            placeholder='you@example.com'
                            {...field}
                          />
                        </FormControl>
                      </FormItem>
                    )}
                  />
                  <FormField
                    control={signUpForm.control}
                    name='password'
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel
                          className={cn('text-xl', formLabelClassName)}
                          {...formLabelProps}
                        >
                          Password
                        </FormLabel>
                        <FormControl>
                          <Input
                            type='password'
                            placeholder='Your password'
                            {...field}
                          />
                        </FormControl>
                      </FormItem>
                    )}
                  />
                  <FormField
                    control={signUpForm.control}
                    name='confirmPassword'
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel
                          className={cn('text-xl', formLabelClassName)}
                          {...formLabelProps}
                        >
                          Confirm Passsword
                        </FormLabel>
                        <FormControl>
                          <Input
                            type='password'
                            placeholder='Confirm your password'
                            {...field}
                          />
                        </FormControl>
                      </FormItem>
                    )}
                  />
                  {statusMessage &&
                    (statusMessage.includes('Error') ||
                    statusMessage.includes('error') ||
                    statusMessage.includes('failed') ||
                    statusMessage.includes('invalid') ? (
                      <StatusMessage message={{ error: statusMessage }} />
                    ) : (
                      <StatusMessage message={{ success: statusMessage }} />
                  ))}
                  <SubmitButton
                    disabled={loading}
                    pendingText='Signing Up...'
                    className={cn(
                      'text-lg font-semibold w-2/3 mx-auto',
                      submitButtonProps?.className
                    )}
                    {...submitButtonProps}
                  >
                    Sign Up
                  </SubmitButton>
                </form>
              </Form>
              <div className='flex items-center w-full gap-4 mt-4'>
                <Separator className='flex-1 bg-accent py-0.5' />
                <span className='text-sm text-muted-foreground'>or</span>
                <Separator className='flex-1 bg-accent py-0.5' />
              </div>
              <SignInWithMicrosoft
                submitButtonProps = {{
                  className: cn(
                    'flex w-5/6 m-auto',
                    signInWithMicrosoftProps?.submitButtonProps?.className),
                }}
                textClassName={cn(
                  'text-lg',
                  signInWithMicrosoftProps?.textClassName,
                )}
                iconClassName={cn(
                  'size-6',
                  signInWithMicrosoftProps?.iconClassName,
                )}
                {...signInWithMicrosoftProps}
              />
              <SignInWithApple
                submitButtonProps = {{
                  className: cn(
                    'flex w-5/6 m-auto',
                    signInWithAppleProps?.submitButtonProps?.className),
                }}
                textClassName={cn(
                  'text-lg',
                  signInWithAppleProps?.textClassName,
                )}
                iconClassName={cn(
                  'size-6',
                  signInWithAppleProps?.iconClassName,
                )}
                {...signInWithAppleProps}
              />
            </CardContent>
          </Card>
        </TabsContent>
      </Tabs>
    </Card>
  );
};

