import { type SupabaseClient, type Profile } from '@/utils/supabase';

type GetStorageProps = {
  client: SupabaseClient;
  bucket: string;
  path: string;
  seconds?: number;
  transform?: {
    width?: number;
    height?: number;
    quality?: number;
    format?: 'origin';
    resize?: 'cover' | 'contain' | 'fill';
  };
  download?: boolean | string;
};

type UploadStorageProps = {
  client: SupabaseClient;
  bucket: string;
  path: string;
  file: File;
  options?: {
    cacheControl?: string;
    contentType?: string;
    upsert?: boolean;
  };
};

type ResizeImageProps = {
  file: File;
  options?: {
    maxWidth?: number;
    maxHeight?: number;
    quality?: number;
  };
};

const getPublicUrl = ({
  client,
  bucket,
  path,
  transform = {},
  download = false,
}: GetStorageProps) => {
  return client.storage
    .from(bucket)
    .getPublicUrl(path, { download, transform});
};

const getSignedUrl = ({
  client,
  bucket,
  path,
  seconds = 3600,
  transform = {},
  download = false,
}: GetStorageProps) => {
  return client.storage
    .from(bucket)
    .createSignedUrl(path, seconds, { download, transform});
};

const uploadFile = ({
  client,
  bucket,
  path,
  file,
  options = {},
}: UploadStorageProps) => {
  return client.storage
    .from(bucket)
    .upload(path, file, options);
};

const updateFile = ({
  client,
  bucket,
  path,
  file,
  options = {
    upsert: true,
  },
}: UploadStorageProps) => {
  return client.storage
    .from(bucket)
    .update(path, file, options);
};

const deleteFiles = ({
  client,
  bucket,
  path,
}: {
  client: SupabaseClient;
  bucket: string;
  path: string[];
  }) => {
  return client.storage.from(bucket).remove(path);
};

const listFiles = ({
  client,
  bucket,
  path = '',
  options = {},
}: {
  client: SupabaseClient;
  bucket: string;
  path?: string;
  options?: {
      limit?: number;
      offset?: number;
      sortBy?: { column: string, order: 'asc' | 'desc' };
  };
}) => {
  return client.storage.from(bucket).list(path, options);
};

const resizeImage = async ({
  file,
  options = {
    maxWidth: 800,
    maxHeight: 800,
    quality: 0.8,
  },
}: ResizeImageProps): Promise<File> => {
  return new Promise((resolve) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = (event) => {
      const img = new Image();
      img.src = event.target?.result as string;
      img.onload = () => {
        let width = img.width;
        let height = img.height;
        if (width > height) {
          if (width > (options.maxWidth ?? 800)) {
            height = Math.round((height * (options.maxWidth ?? 800)) / width);
            width = options.maxWidth ?? 800;
          }
        } else if (height > (options.maxHeight ?? 800)) {
          width = Math.round((width * (options.maxHeight ?? 800)) / height);
          height = options.maxHeight ?? 800;
        }
        const canvas = document.createElement('canvas');
        canvas.width = width;
        canvas.height = height;
        const ctx = canvas.getContext('2d');
        ctx?.drawImage(img, 0, 0, width, height);
        canvas.toBlob(
          (blob) => {
            if (!blob) return;
            const resizedFile = new File([blob], file.name, {
              type: 'imgage/jpeg',
              lastModified: Date.now(),
            });
            resolve(resizedFile);
          },
          'image/jpeg',
          (options.quality && options.quality < 1 && options.quality > 0)
            ? options.quality
            : 0.8,
        );
      };
    };
  });
};

export {
  deleteFiles,
  getPublicUrl,
  getSignedUrl,
  listFiles,
  resizeImage,
  uploadFile,
  updateFile
};
